@@ -179,6 +179,7 @@ class Script:
             "pty",
             "input"
             "check",
+            "pool_size",
         ]
 
         if not hosts:
@@ -187,9 +188,13 @@ class Script:
         run_kwargs["hosts"] = hosts
         run_kwargs.update({k:v for k, v in self.run_args.items() if k in whitelist})
 
+        ignore_cli = (
+            "files", # we use "files" to copy our script, so we need to merge instead or replace
+            "prefix", # if the script manages this value, it is unlikely to work with prefixes
+        )
 
         # Make sure we "upload ourself", so we can just run the script
-        run_kwargs.update({k:v for k, v in kwargs.items() if v is not None and k != "files"})
+        run_kwargs.update({k:v for k, v in kwargs.items() if v is not None and k not in ignore_cli})
         run_kwargs.setdefault("files", [])
         run_kwargs["files"].extend(kwargs.get("files", []))
         run_kwargs["files"].append(self.path)
@@ -208,8 +213,8 @@ class PasswordManager:
     def _prompt(message):
         return _click.prompt(message, hide_input=True, err=True)
 
-    def get(self, message):
-        if self._password is None:
+    def get(self, message=None):
+        if message and self._password is None:
             self._password = self._prompt(message)
 
         return self._password
@@ -246,6 +251,7 @@ class Task:
         cwd=None,
         preexec_fn=None,
         postexec_fn=None,
+        universal_newlines=None,
     ): # pylint: disable=too-many-locals
         if encoding or errors:
             text = True
@@ -260,6 +266,9 @@ class Task:
         if pty and (input or stdin):
             raise TuesError("Passing `input` or `stdin` in `pty` mode is not supported")
 
+        if pty and universal_newlines is None:
+            universal_newlines = True
+
         # `asyncssh.connect` wants an empty tuple over `None` as default
         if login_user is None:
             login_user = ()
@@ -294,6 +303,7 @@ class Task:
         self.prefix_width_hint = prefix_width_hint or len(self.host)
         self.user = user
         self.pty = pty
+        self.universal_newlines = universal_newlines if pty else False
         self.input = input
         self.stdin = stdin
         self.text = text
@@ -485,17 +495,19 @@ class SudoWriter:
     # No idea why this token it not configurable in sudo, but since we are
     # making our own success token, we do not really need this, we only keep
     # it around so we are able to clean it from the datastream
-    FAILURE_TOKEN = b"Sorry, try again.\r\n"
 
     def __init__(self, f, run, pm, on_success=None):
         self._f = f
         self.sudo = None
         self._pm = pm
+        self.newline = b"\r\n" if run.universal_newlines else b"\n"
+        self.failure_token = b"Sorry, try again." + self.newline
         self._post_prompt_filter = (
             self.SUCCESS_TOKEN,
-            self.FAILURE_TOKEN,
-            b"\r\n",
+            self.failure_token,
+            self.newline,
         )
+
         self._run = run
         self._on_success = on_success
         self._waiting = False
@@ -503,15 +515,17 @@ class SudoWriter:
         run.cmdwrapper = lambda cmd: f"sudo -S -u {_shlex.quote(run.user)} -p {_shlex.quote(self.PROMPT_TOKEN.decode())} -- bash -c {_shlex.quote(cmd)}"
 
     async def write(self, data):
-        if self.FAILURE_TOKEN in data or self.SUCCESS_TOKEN in data:
+        failure_cond = self.failure_token in data
+        success_cond = self.SUCCESS_TOKEN in data
+        if failure_cond or success_cond:
             _log.debug("SudoWriter found stop token in %r", data)
-            if self.FAILURE_TOKEN in data:
+            # Only invalidate if nobody else has reprompted a password yet
+            if failure_cond and self._last_pw_attempted == self._pm.get():
                 self._pm.invalidate()
 
-            if self.SUCCESS_TOKEN in data:
-                if self._on_success:
-                    await self._on_success()
-                    self._on_success = None
+            if success_cond and self._on_success:
+                await self._on_success()
+                self._on_success = None
 
         if self._waiting:
             for filter_ in self._post_prompt_filter:
@@ -524,8 +538,8 @@ class SudoWriter:
         if self.PROMPT_TOKEN in data:
             data = data.replace(self.PROMPT_TOKEN, b"")
             self._waiting = True
-            password = self._pm.get("Your remote sudo password")
             _log.debug("SudoWriter prompt reply on %r", self.sudo)
+            self._last_pw_attempted = password = self._pm.get("Your remote sudo password")
             self.sudo.write((password + "\n").encode())
             await self.sudo.drain()
 
@@ -633,9 +647,9 @@ async def _redirect_io(session, stdout, stderr, sudo):
     if sudo and not isinstance(sudo, OutputWrapper):
         sudo = OutputWrapper(sudo)
 
-    _log.debug("STDOUT %r", stdout)
-    _log.debug("STDERR %r", stderr)
-    _log.debug("SUDO %r", sudo)
+    _log.debug("IO Handle for stdout set to %r", stdout)
+    _log.debug("IO Handle for stderr set to  %r", stderr)
+    _log.debug("IO Handle for sudo set  %r", sudo)
 
     await session.redirect(stdin=None, stdout=stdout, stderr=stderr or stdout)
 
@@ -735,6 +749,7 @@ async def _run(run, pm, stdout=None, stderr=None): # pylint: disable=too-many-lo
             _ssh.SSHClientProcess,
             cmd,
             term_type=_os.environ.get("TERM"),
+            term_modes={_ssh.PTY_ONLCR: 0, _ssh.PTY_INLCR: 0} if not run.universal_newlines else (),
             request_pty=run.pty,
             env=env,
             # If we don't "force" them to None, some kind of autodetection will take place
@@ -779,7 +794,7 @@ async def _wait_with_concurrency(tasks, pool_size):
     async def sem_task(task):
         async with semaphore:
             return await task
-    return await _asyncio.wait([sem_task(task) for task in tasks])
+    return await _asyncio.wait([_asyncio.create_task(sem_task(task)) for task in tasks])
 
 
 async def run_tasks(tasks, pm=_PM, stdout=None, stderr=None, pool_size=1): # pylint: disable=too-many-locals
@@ -871,6 +886,7 @@ def run(
     preexec_fn=None,
     postexec_fn=None,
     output_dir_strategy=DIR_IGNORE,
+    universal_newlines=None,
 ): # pylint: disable=too-many-locals
     """
         Run `cmd` on all `hosts`.
@@ -973,6 +989,11 @@ def run(
                 In this mode you will not be able to distinguish between stdout
                 and stderr, all output will seem to be coming from stdout.
 
+            universal_newlines (bool): When pty mode is enabled, \n will be automtically
+                converted to \r\n in either direction, even when dealing with bytes.
+                To prevent the pty layer from manipulating the bytestream, you can
+                set `universal_newlines` to `False`.
+
             login_user (str): The user to log-in with, usual system defaults
                 and ssh config settings apply, so you should only very rarely
                 need this option.
@@ -1008,6 +1029,7 @@ def run(
             prefix_width_hint=host_width if align_prefix else None,
             user=user,
             pty=pty,
+            universal_newlines=universal_newlines,
             input=input,
             text=text,
             errors=errors,
@@ -1025,7 +1047,7 @@ def run(
         try:
             loop = _asyncio.get_running_loop()
         except RuntimeError:
-            loop = _asyncio.get_event_loop()
+            loop = _asyncio.new_event_loop()
 
     try:
         if pool_size > 1:
